#!/usr/bin/env python3

"""
Supported instruments (identified): newport_1918c
- 
"""
import time
import ctypes as ct

class Driver():
    
    def __init__(self):
        self.write('PM:DS:Clear') # Clear data store
        self.write('PM:AUTO 1') # Automatic range
        self.write('PM:UNITs 2') # for Watts
    
    def get_id(self):
        return self.query('*IDN?')
    
    
    def zero(self):
        self.write('PM:ZEROSTOre')
        
        
        
    def set_auto_range(self,value):
        self.write(f'PM:AUTO {int(value)}')
        
    def get_auto_range(self):
        return bool(int(self.query('PM:AUTO?')))
        
        
        
        
    def set_zero_value(self,value):
        self.write(f'PM:ZEROVALue {float(value):.5f}')
        
    def get_zero_value(self):
        return float(self.query('PM:ZEROVALue?'))




    def set_wavelength(self, wavelength):     
        self.write(f'PM:Lambda {int(wavelength)}')

    def get_wavelength(self):
        return float(self.query('PM:Lambda?'))






    def set_buffer_size(self,value):
        self.write(f'PM:DS:SIZE {int(value)}')
    
    def get_buffer_size(self):
        return float(self.query('PM:DS:SIZE?'))
    
    
    
    

    def set_buffer_interval(self,value): #ms
        self.write(f'PM:DS:INT {int(value)*10}') 
    
    def get_buffer_interval(self):
        return float(self.query('PM:DS:INT?'))/10 
    
    
    def get_power_mean(self):

        """
        Stores the power values at a certain wavelength.
        """
        
        # On vide la mémoDL
        self.write('PM:DS:Clear')
        
        # On lance l'acquisition
        self.write('PM:DS:ENable 1')
        
        # On attend la fin de l'acquisition
        buffer_size = self.get_buffer_size()
        buffer_interval = self.get_buffer_interval()
        while int(self.query('PM:DS:Count?')) < buffer_size :
            time.sleep(buffer_interval)
        
        # On récupère les résultats de mesure
        mean=float(self.query('PM:STAT:MEAN?'))
        return mean        
    

    def get_power(self):
        while True :
            power = float(self.query('PM:Power?'))
            if power < 100 :
                break
            else :
                time.sleep(0.1)
        return power
        

    def get_driver_model(self):
        
        model = []
        
        model.append({'element':'action','name':'zero','do':self.zero,                       
                       'help':'Sets the zeroing value with the present reading.'})

        model.append({'element':'variable','name':'autorange','type':bool,
                       'read':self.get_auto_range,'write':self.set_auto_range,
                       'help':'Auto range enable command.'})

        model.append({'element':'variable','name':'zero_value','type':float,
                       'read':self.get_zero_value,'write':self.set_zero_value,
                       'help':'Sets the zeroing value.'})

        model.append({'element':'variable','name':'buffer_size','type':int,
                       'read':self.get_buffer_size,'write':self.set_buffer_size,
                       'help':'Sets the size of the Data Store buffer.'})    

        model.append({'element':'variable','name':'buffer_interval','type':int,
                       'read':self.get_buffer_interval,'write':self.set_buffer_interval,
                       'help':'Set data store interval.'})   
    
        model.append({'element':'variable','name':'wavelength','type':float,
                       'read':self.get_wavelength,'write':self.set_wavelength,
                       'help':'Sets the wavelength for use when calculating power.'})      

        model.append({'element':'variable','name':'power','type':float,
                       'read':self.get_power,
                       'help':'Returns the power in the selected units.'})      
    
        model.append({'element':'variable','name':'power_mean','type':float,
                       'read':self.get_power_mean, 'help':'Mean power value at a certain wavelength.'})    
    
        return model


#################################################################################
############################## Connections classes ##############################
class Driver_DLL(Driver):
    
    def __init__(self,libpath=r'C:\Program Files\Newport\Newport USB Driver\Bin\usbdll.dll',**kwargs):
        
        self.productID=0xCEC7
        self.modelNumber=1918
        
        self.controller = ct.windll.LoadLibrary(libpath)
        
        # Open device
        cproductid = ct.c_int(self.productID)
        useusbaddress = ct.c_bool(1)
        num_devices = ct.c_int()
        status = self.controller.newp_usb_open_devices(cproductid, useusbaddress, ct.byref(num_devices))

        # Test ouverture
        if status != 0 or num_devices.value == 0:
            raise ValueError('Impossible to load the device')

        # Test device information
        arInstruments = ct.c_int()
        arInstrumentsModel = ct.c_int()
        arInstrumentsSN = ct.c_int()
        nArraySize = ct.c_int()
        status = self.controller.GetInstrumentList(ct.byref(arInstruments), 
                                                   ct.byref(arInstrumentsModel), 
                                                   ct.byref(arInstrumentsSN),
                                                   ct.byref(nArraySize))
        if status != 0 :
            raise ValueError('Impossible to load device information')

        # Test model number
        if arInstrumentsModel.value != self.modelNumber:
            raise ValueError('The device model number is not correct for this driver')
            
        self.IDnum = arInstruments.value
            
        Driver.__init__(self)
        
        
        
    def close(self):
        try: self.controller.newp_usb_uninit_system()
        except : pass

    def query(self,command):
        self.write(command)
        return self.read()


    def read(self):  
        try :         
            response = ct.create_string_buffer(1024)
            length = ct.c_ulong(1024)
            read_bytes = ct.c_ulong()
            cdevice_id = ct.c_long(self.IDnum)
            status = self.controller.newp_usb_get_ascii(cdevice_id, ct.byref(response), length, ct.byref(read_bytes))
            if status == 0:
                answer = response.value[0:read_bytes.value].decode().rstrip('\r\n')
                return answer
        except :
            pass

    def write(self, commandString):
        try :
            assert isinstance(commandString,str)
            query = ct.create_string_buffer(commandString.encode())
            length = ct.c_ulong(ct.sizeof(query))
            cdevice_id = ct.c_long(self.IDnum)
            self.controller.newp_usb_send_ascii(cdevice_id, ct.byref(query), length)
        except:
            pass
############################## Connections classes ##############################
#################################################################################



# -*- coding: utf-8 -*-
"""
Created on Sun Sep 29 18:15:26 2019

@author: qchat
"""

from PyQt5 import QtCore, QtWidgets, QtGui
from . import main 

class MyQTreeWidget(QtWidgets.QTreeWidget):
    
    reorderSignal = QtCore.pyqtSignal(object) 
    
    def __init__(self,parent):
        
        QtWidgets.QTreeWidget.__init__(self,parent)
    
    def dropEvent(self, event):
        QtWidgets.QTreeWidget.dropEvent(self, event)
        self.reorderSignal.emit(event)


class RecipeManager :
    
    def __init__(self,gui):
        
        self.gui = gui
        
        # Tree configuration
        #self.tree = self.gui.recipe_treeWidget
        self.tree = MyQTreeWidget(self.gui)
        self.gui.tree_layout.addWidget(self.tree)
        spacer = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gui.tree_layout.addItem(spacer)
        self.tree.setHeaderLabels(['Step name','Type','Element address','Value'])
        self.tree.header().resizeSection(3, 50)
        self.tree.itemDoubleClicked.connect(self.itemDoubleClicked)
        self.tree.reorderSignal.connect(self.orderChanged)
        self.tree.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
        self.tree.setDropIndicatorShown(True)
        self.tree.setAlternatingRowColors(True)        
        self.tree.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tree.setMinimumSize(450,500)
        self.tree.setSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        self.tree.customContextMenuRequested.connect(self.rightClick)
                
        self.defaultItemBackground = None
        
        
    def orderChanged(self,event):
        newOrder = [self.tree.topLevelItem(i).text(0) for i in range(self.tree.topLevelItemCount())]
        self.gui.configManager.setRecipeOrder(newOrder)
        

    def refresh(self):
        
        """ Refresh the whole scan recipe displayed from the configuration center """
        
        self.tree.clear()
        
        recipe = self.gui.configManager.getRecipe()

        for i in range(len(recipe)):
            
            # Loading step informations
            step = recipe[i]
            item = QtWidgets.QTreeWidgetItem()
            item.setFlags(item.flags() ^ QtCore.Qt.ItemIsDropEnabled ) 
            
            # Column 1 : Step name
            item.setText(0,step['name'])
            
            # Column 2 : Step type
            if step['stepType'] == 'measure' : 
                item.setText(1,'Measure variable')
            elif step['stepType']  == 'set' : 
                item.setText(1,'Set variable')
            elif step['stepType']  == 'action' : 
                item.setText(1,'Do action')
                            
            # Column 3 : Element address
            item.setText(2,step['element'].address())
            
            # Column 4 : Value if stepType is 'set'
            value = step['value']
            if value is not None : 
                if step['element'].type in [bool,str] :
                   item.setText(3,f'{value}')
                else :
                   item.setText(3,f'{value:g}')
            
            # Add item to the tree
            self.tree.addTopLevelItem(item)
            self.defaultItemBackground = item.background(0)
            
            
            
    def rightClick(self,position):
        
        """ This functions provide a menu where the user right clicked """
    
        item = self.tree.itemAt(position)
        
        if item is not None and self.gui.scanManager.isStarted() is False :
            
            name = item.text(0)
            stepType = self.gui.configManager.getRecipeStepType(name)
            element = self.gui.configManager.getRecipeStepElement(name)
                        
            menuActions = {}
            
            menu = QtWidgets.QMenu()         
            menuActions['rename'] = menu.addAction("Rename")
            if stepType == 'set' or (stepType == 'action' and element.type in [int,float,str]) :
                menuActions['setvalue'] = menu.addAction("Set value")
            menuActions['remove'] = menu.addAction("Remove")
            
            choice = menu.exec_(self.tree.viewport().mapToGlobal(position))
            
            if 'rename' in menuActions.keys() and choice == menuActions['rename'] :
                self.renameStep(name)                
            elif 'remove' in menuActions.keys() and choice == menuActions['remove'] :
                self.gui.configManager.delRecipeStep(name)
            elif 'setvalue' in menuActions.keys() and choice == menuActions['setvalue'] :
                self.setStepValue(name)

        
        
    def renameStep(self,name):
        
        """ This function prompts the user for a new step name, and apply it to the selected step """
        
        newName,state = QtWidgets.QInputDialog.getText(self.gui, name, f"Set {name} new name",
                                                 QtWidgets.QLineEdit.Normal, name)
        
        newName = main.cleanString(newName)
        if newName != '' :
            self.gui.configManager.renameRecipeStep(name,newName)
            
   
            
    def setStepValue(self,name):
        
        """ This function prompts the user for a new step value, and apply it to the selected step """
        
        element = self.gui.configManager.getRecipeStepElement(name)
        value = self.gui.configManager.getRecipeStepValue(name)
        
        # Default value displayed in the QInputDialog
        if element.type in [str] :
            defaultValue = f'{value}'
        else :
            defaultValue = f'{value:g}'
        
        value,state = QtWidgets.QInputDialog.getText(self.gui, 
                                                     name, 
                                                     f"Set {name} value",
                                                     QtWidgets.QLineEdit.Normal, defaultValue)
        
        if value != '' :
            
            try :   
                
                # Type conversions
                if element.type in [int]:
                    value = int(value)
                elif element.type in [float] :
                    value = float(value)
                elif element.type in [str] :
                    value = str(value)
                elif element.type in [bool]:
                    value = int(value)
                    assert value in [0,1]
                    value = bool(value)
                
                # Apply modification
                self.gui.configManager.setRecipeStepValue(name,value)
                
            except :
                pass
            
            
            
    def itemDoubleClicked(self,item,column):
        
        """ This function executes an action depending where the user double clicked """
        
        name = item.text(0)
        stepType = self.gui.configManager.getRecipeStepType(name)
        element = self.gui.configManager.getRecipeStepElement(name)
        
        if column == 0 : 
            self.renameStep(name)
        if column == 3 :
            if stepType == 'set' or (stepType == 'action' and element.type in [int,float,str]) :
                self.setStepValue(name)
   
    

    def setStepProcessingState(self,name,state):
        
        """ This function set the background color of a recipe step during the scan """
        
        item = self.tree.findItems(name, QtCore.Qt.MatchExactly, 0)[0]
        
        if state is None :
            item.setBackground(0,self.defaultItemBackground)
        if state == 'started' :
            item.setBackground(0, QtGui.QColor('#ff8c1a'))
        elif state == 'finished' :
            item.setBackground(0, QtGui.QColor('#70db70'))
                                               
        
        
    def resetStepsProcessingState(self):
        
        """ This function reset the background color of a recipe once the scan is finished """
        
        for name in self.gui.configManager.getNames(option='recipe'):
            self.setStepProcessingState(name,None)
            

#
#class Step:
#    
#    def __init__(self,name):
#        self.name = None
#        
#    def set_name(self,name):
#        self.name = name
#        
#    def message(self,mess):
#        return f'Step {self.name} (self.step_type): {mess}'
#        
#
#class ExecuteActionStep(Step):
#    step_type = 'execute_action'
#    
#    def __init__(self,name,element,value=None):
#        Step.__init__(self,name)
#        
#        # Element
#        assert element._element_type == 'action', self.message('The element has to be an Action')
#        self.action = element
#        
#        # Value
#        if self.action.has_parameter : 
#            assert value is not None, self.message('Parameter value required')
#            self.set_value(value) 
#        else :
#            assert value is None, self.message('This action has no parameter')
#            
#    def set_value(self,value):
#        assert self.action.has_parameter, self.message('This action has no parameter')
#        try : 
#            self.value = self.action.type(value)
#        except : 
#            raise ValueError(self.message(f'Impossible to convert {value} in type {self.element.type.__name__}'))
#        
#    def get_value(self):
#        return self.value
#    
#    def run(self):
#        if self.element.has_parameter : 
#            assert self.value is not None 
#            self.action(self.value)
#        else : 
#            self.action()
#        
#
#    
#class ScanVariableStep:
#    step_type = 'scan_variable'
#    
#    def __init__(self,name,element):
#        
#        Step.__init__(self,name)
#        
#        # Element
#        assert element._element_type == 'variable', self.message('The element has to be a Variable')
#        assert element.writable is True, self.message('The Variable has to be writable')
#        self.variable = element
#        
#        
#class ScanVariableEndStep:
#    step_type = 'scan_variable_end'
#    
#    def __init__(self,name):
#        
#        
#    def run(self):
#        
#        
#        
#class MeasureVariableStep:
#    step_type = 'measure_variable'
#    
#    def __init__(self,name,element):
#        Step.__init__(self,name)
#        
#        # Element
#        assert element._element_type == 'variable', self.message('The element has to be a Variable')
#        assert element.readable is True, self.message('The Variable has to be readable')
#        self.variable = element
#        
#        
#    def run(self):
#        return self.variable()
#        
#        
#        
#    
#class SetVariableStep:
#    step_type = 'set_variable'
#    
#    def __init__(self,name,element,value):
#        Step.__init__(self,name)
#        
#        # Element
#        assert element._element_type == 'variable', self.message('The element has to be a Variable')
#        assert element.writable is True, self.message('The Variable has to be writable')
#        self.variable = element
#        
#        # Value
#        self.set_value(value)
#        
#    
#    def set_value(self,value):
#        try : 
#            self.value = self.variable.type(value)
#        except : 
#            raise ValueError(self.message(f'Impossible to convert {value} in type {self.element.type.__name__}'))
#        
#    def get_value(self):
#        return self.value
#        
#    def run(self):
#        self.variable(self.value)
#        
#    
#    def prompt_value(self):
#            
#        # Default value displayed in the QInputDialog
#        if self.variable.type == str :
#            defaultValue = f'{self.value}'
#        else :
#            defaultValue = f'{self.value:g}'
#        
#        value,state = QtWidgets.QInputDialog.getText(self.gui, 
#                                                     name, 
#                                                     f"Set {name} value",
#                                                     QtWidgets.QLineEdit.Normal, defaultValue)
#        
#        if value != '' :
#            
#            try :   
#                
#                # Type conversions
#                if element.type in [int]:
#                    value = int(value)
#                elif element.type in [float] :
#                    value = float(value)
#                elif element.type in [str] :
#                    value = str(value)
#                elif element.type in [bool]:
#                    value = int(value)
#                    assert value in [0,1]
#                    value = bool(value)
#                
#                # Apply modification
#                self.gui.configManager.setRecipeStepValue(name,value)
#                
#            except :
#                pass
#        
        
        
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- Keopsys CEFA-C-PB-HP
"""




class Driver() :
    
    def __init__(self):
        pass
        
    def get_id(self):
        return self.query('*IDN?')
    
    def set_power(self, value):             # For this model range is from 20dBm to 30dBm, 200=20dBm here 
        self.write(f"CPU="+str(value))
        
    def get_power(self):
        return float(self.query('CPU?').split('=')[1])
    
    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'power','type':float,'read':self.get_power,'write':self.set_power, 'help':'Set power.'})
        return model
    
#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB0::2::INSTR', **kwargs):
        import visa
        
        rm = visa.ResourceManager()
        self.controller = rm.open_resource(address)
        self.controller.write_termination = 0x00  #needed in order to read properly from the optical amplifier 
        self.controller.read_termination = 0x00

        Driver.__init__(self)

    def close(self):
        self.controller.close()
        
    def query(self,command):
        result = self.controller.query(command) 
        result = result.strip('\x00')
        return result

    def write(self,command):
        self.controller.write(command)
############################## Connections classes ##############################
#################################################################################


